import json
import logging
import six
from ipalib import api, errors, _
from ipalib import Command, Str, Int
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (baseuser, baseuser_output_params, validate_nsaccountlock)
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPAddMember, LDAPRemoveMember)
from ipalib import ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
MAC categories
""")

register = Registry()

user_output_params = baseuser_output_params

_pcat_flags_map = {
    '0': '1',
    '1': '2',
    '2': '4',
    '3': '8',
    '4': '10',
    '5': '20',
    '6': '40',
    '7': '80',
    '8': '100',
    '9': '200',
    '10': '400',
    '11': '800',
    '12': '1000',
    '13': '2000',
    '14': '4000',
    '15': '8000',
    '16': '10000',
    '17': '20000',
    '18': '40000',
    '19': '80000',
    '20': '100000',
    '21': '200000',
    '22': '400000',
    '23': '800000',
    '24': '1000000',
    '25': '2000000',
    '26': '4000000',
    '27': '8000000',
    '28': '10000000',
    '29': '20000000',
    '30': '40000000',
    '31': '80000000',
    '32': '100000000',
    '33': '200000000',
    '34': '400000000',
    '35': '800000000',
    '36': '1000000000',
    '37': '2000000000',
    '38': '4000000000',
    '39': '8000000000',
    '40': '10000000000',
    '41': '20000000000',
    '42': '40000000000',
    '43': '80000000000',
    '44': '100000000000',
    '45': '200000000000',
    '46': '400000000000',
    '47': '800000000000',
    '48': '1000000000000',
    '49': '2000000000000',
    '50': '4000000000000',
    '51': '8000000000000',
    '52': '10000000000000',
    '53': '20000000000000',
    '54': '40000000000000',
    '55': '80000000000000',
    '56': '100000000000000',
    '57': '200000000000000',
    '58': '400000000000000',
    '59': '800000000000000',
    '60': '1000000000000000',
    '61': '2000000000000000',
    '62': '4000000000000000',
    '63': '8000000000000000'
}

def mod_min_pcat(user_pcat):

    mod_pcat = str(user_pcat)
    x1 = mod_pcat.find('x')
    x2 = mod_pcat.find(':', x1)
    mod_pcat = list(mod_pcat)
    mod_pcat = mod_pcat[x1+1:x2]
    mod_pcat = ''.join(mod_pcat)
    mod_pcat = str(mod_pcat)

    return mod_pcat

def mod_max_pcat(user_pcat):

    mod_pcat = str(user_pcat)
    x1 = mod_pcat.find('x')
    mod_pcat = list(mod_pcat)
    mod_pcat = mod_pcat[x1+1:]
    mod_pcat = ''.join(mod_pcat)
    mod_pcat = str(mod_pcat)

    return mod_pcat

def pcat_hex(sumpcat):

    pcat = (hex(sumpcat))
    pcat = list(pcat)
    pcat[:2] = []
    pcat = ''.join(pcat)

    return (pcat)

def add_macmincat_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberuser in list(options.get('user', [])):

              user = memberuser

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

              fuser_attrs = "/tmp/fuser_attrs.json"
              with open(fuser_attrs, "wb") as f:
                 json.dump(list(user_attrs), f)

              user_pcat = user_attrs['x-ald-user-mac'][0]

#              mod_pcat = mod_min_pcat(user_pcat)

              mod_pcat = str(user_pcat)
              x1 = mod_pcat.find('x')
              x2 = mod_pcat.find(':', x1)
              mod_pcat = list(mod_pcat)
              mod_pcat = mod_pcat[x1+1:x2]
              mod_pcat = ''.join(mod_pcat)
              mod_pcat = str(mod_pcat)

              sumpcat = '0'
              for name in keys:
                 sumpcat = int(_pcat_flags_map.get(name), 16) + int(mod_pcat, 16)

                 pcat = pcat_hex(sumpcat)

              user_pcat = user_pcat[:x1+1] + pcat + user_pcat[x2:]
              user_attrs.update({'x-ald-user-mac': user_pcat})

        except:
          logging.exception("pcat_update:")

        try:

           keys = [user]
           kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}
           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")

def mov_macmincat_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberof in list(options.get('user', [])):
              user = memberof

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

           if 'x-ald-user-mac' in user_attrs:
             user_pcat = user_attrs['x-ald-user-mac'][0]

#             mod_pcat = mod_min_pcat(user_pcat)

             mod_pcat = str(user_pcat)
             x1 = mod_pcat.find('x')
             x2 = mod_pcat.find(':', x1)
             mod_pcat = list(mod_pcat)
             mod_pcat = mod_pcat[x1+1:x2]
             mod_pcat = ''.join(mod_pcat)
             mod_pcat = str(mod_pcat)

             sumpcat = '0'
             for name in keys:
                sumpcat = int(mod_pcat, 16) - int(_pcat_flags_map.get(name), 16)

                pcat = pcat_hex(sumpcat)

             user_pcat = user_pcat[:x1+1] + pcat + user_pcat[x2:]
             user_attrs.update({'x-ald-user-mac': user_pcat})

        except:
          logging.exception("pcat_update:")

        try:

           keys = [user]

           kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}

           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")

def add_macmaxcat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberuser in list(options.get('user', [])):

              user = memberuser

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

              fuser_attrs = "/tmp/fuser_attrs.json"
              with open(fuser_attrs, "wb") as f:
                 json.dump(list(user_attrs), f)

              user_pcat = user_attrs['x-ald-user-mac'][0]

              fuser_pcat = "/tmp/fuser_pcat.json"
              with open(fuser_pcat, "wb") as f:
                 json.dump(user_pcat, f)

              mod_pcat = str(user_pcat)
              x1 = mod_pcat.rfind('x')
              mod_pcat = list(mod_pcat)
              mod_pcat = mod_pcat[x1+1:]
              mod_pcat = ''.join(mod_pcat)
              mod_pcat = str(mod_pcat)

              sumpcat = '0'
              for name in keys:
                 sumpcat = int(_pcat_flags_map.get(name), 16) + int(mod_pcat, 16)

                 fsumpcat = "/tmp/fsumpcat.json"
                 with open(fsumpcat, "wb") as f:
                    json.dump(sumpcat, f)

                 pcat = pcat_hex(sumpcat)

              user_pcat = user_pcat[:x1+1] + pcat

              fuser_pcat = "/tmp/fuser_pcat.json"
              with open(fuser_pcat, "wb") as f:
                  json.dump(user_pcat, f)

              user_attrs.update({'x-ald-user-mac': user_pcat})

        except:
          logging.exception("pcat_update:")

        try:

           keys = [user]
           kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}
           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")

def mov_pcat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberof in list(options.get('user', [])):
              user = memberof

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

           if 'x-ald-user-mac' in user_attrs:
             user_pcat = user_attrs['x-ald-user-mac'][0]

#             mod_pcat = mod_max_pcat(user_pcat)
             mod_pcat = str(user_pcat)
             x1 = mod_pcat.rfind('x')
             mod_pcat = list(mod_pcat)
             mod_pcat = mod_pcat[x1+1:]
             mod_pcat = ''.join(mod_pcat)
             mod_pcat = str(mod_pcat)

             sumpcat = '0'
             for name in keys:
                sumpcat = int(mod_pcat, 16) - int(_pcat_flags_map.get(name), 16)

                pcat = pcat_hex(sumpcat)

             user_pcat = user_pcat[:x1+1] + pcat

             user_attrs.update({'x-ald-user-mac': user_pcat})

        except:
          logging.exception("pcat_update:")

        try:

           keys = [user]

           kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}

           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")

def get_dn(self, *keys, **kwargs):

    try:
        if keys[-1] is not None:
           return self.api.Object.user.get_dn(*keys, **kwargs)
    except:
          logging.exception("get_dn:")
    return DN(self.container_dn, api.env.basedn)

topic = 'macmincat'

@register()
class macmincat(LDAPObject):
    """
    Levels min categories
    """
    container_dn = DN(('cn', 'macmincat'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('macmincat')
    object_name_plural = _('macmincats')
    object_class = ['x-ald-mac-category-object']
    default_attributes = ['cn', 'x-ald-mac-category', 'member']
    search_attributes = ['cn', 'x-ald-mac-category', 'member']

    attribute_members = {
        'member': ['user'],
    }

    label = _('Min category levels')
    label_singular = _('Min category levels')

    takes_params = (
        Str('cn',
            cli_name='macmincat',
            label=_('Min category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            cli_name='pcategoryid',
            label=_('Min category rank'),
            doc=_('integer to order the MAC categorys'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage categories for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class macmincat_add(LDAPCreate):
    __doc__ = _('Create new MAC min category')

    msg_summary = _('Added MAC min category"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        try:
            # check duplicity
            api.Object['x-ald-mac-category'].get_dn_if_exists(keys[-1])
            self.obj.handle_duplicate_entry(*keys)
        except errors.NotFound:
            pass

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        return dn


@register()
class macmincat_del(LDAPDelete):
    __doc__ = _('Delete MAC min category.')

    msg_summary = _('Deleted MAC min category"%(value)s"')


@register()
class macmincat_mod(LDAPUpdate):
    __doc__ = _('Modify MAC min category.')

    msg_summary = _('Modified MAC min category"%(value)s"')


@register()
class macmincat_find(LDAPSearch):
    __doc__ = _('Search for MAC min category.')

    msg_summary = ngettext(
        '%(count)d MAC min category matched',
        '%(count)d MAC category min matched', 0
    )

@register()
class macmincat_show(LDAPRetrieve):
    __doc__ = _('Display MAC min category.')

@register()
class macmincat_add_macmincat(LDAPAddMember):

    __doc__ = _('Modify user min categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_macmincat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class macmincat_remove_macmincat(LDAPRemoveMember):
    __doc__ = _('Remove users min categorys.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_macmincat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

topic = 'macmaxcat'

@register()
class macmaxcat(LDAPObject):
    """
    Levels max categories
    """
    container_dn = DN(('cn', 'macmaxcat'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('macmaxcat')
    object_name_plural = _('macmaxcats')
    object_class = ['x-ald-mac-category-object']
    default_attributes = ['cn', 'x-ald-mac-category', 'member']
    search_attributes = ['cn', 'x-ald-mac-category', 'member']

    attribute_members = {
        'member': ['user'],
    }

    label = _('Max category levels')
    label_singular = _('Max category levels')

    takes_params = (
        Str('cn',
            cli_name='macmaxcat',
            label=_('Max category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            cli_name='pcategoryid',
            label=_('max category rank'),
            doc=_('integer to order the MAC categorys'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage categories for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class macmaxcat_add(LDAPCreate):
    __doc__ = _('Create new MAC max category')

    msg_summary = _('Added MAC max category"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        try:
            # check duplicity
            api.Object['x-ald-mac-category'].get_dn_if_exists(keys[-1])
            self.obj.handle_duplicate_entry(*keys)
        except errors.NotFound:
            pass

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        return dn

@register()
class macmaxcat_del(LDAPDelete):
    __doc__ = _('Delete MAC max category.')

    msg_summary = _('Deleted MAC max category"%(value)s"')

@register()
class macmaxcat_mod(LDAPUpdate):
    __doc__ = _('Modify MAC max category.')

    msg_summary = _('Modified MAC max category"%(value)s"')

@register()
class macmaxcat_find(LDAPSearch):
    __doc__ = _('Search for MAC max category.')

    msg_summary = ngettext(
        '%(count)d MAC max category matched',
        '%(count)d MAC category max matched', 0
    )

@register()
class macmaxcat_show(LDAPRetrieve):
    __doc__ = _('Display MAC max category.')

@register()
class macmaxcat_add_macmaxcat(LDAPAddMember):

    __doc__ = _('Modify user categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_macmaxcat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class macmaxcat_remove_macmaxcat(LDAPRemoveMember):
    __doc__ = _('Remove users categorys.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_pcat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)
